
-- View: "timesheet_import"
-- DROP VIEW timesheet_import;

CREATE OR REPLACE VIEW timesheet_import AS 
SELECT 
	d."Date"::timestamp with time zone AS "day", 
	fr."WorkTime"::numeric(7,4) AS hours, 
	rtrim(e."LoginName") AS username, 
	rtrim(e."MT_Email") AS email, 
        CASE
            WHEN length(fr."ProjectID"::text) >= 8 THEN fr."ProjectID"::text
            ELSE 'I'::text || lpad(fr."ProjectID"::text, 4, '0'::text)
        END AS navision_project_nr, 
	p."InternalProject" AS project_internal
FROM 
	(SELECT	raw."EmployeeID", raw."DateID", raw."ProjectID", sum(raw."WorkTime") AS "WorkTime"
         FROM	timesheet."TS_FactRow" raw
         GROUP BY
		raw."EmployeeID", raw."DateID", raw."ProjectID"
         HAVING
		sum(raw."WorkTime") > 0::numeric
	) fr
	JOIN timesheet."TS_Date" d ON fr."DateID" = d."DateID"
	JOIN timesheet."TS_Employee" e ON fr."EmployeeID" = e."EmployeeID"
	JOIN timesheet."TS_Project" p ON fr."ProjectID" = p."ProjectID"
;



-- Function: timesheet_import_routine()
-- DROP FUNCTION timesheet_import_routine();

CREATE OR REPLACE FUNCTION timesheet_import_routine()
  RETURNS integer AS
$BODY$
DECLARE
	row		RECORD;
	v_user_id	integer;
	v_project_id	integer;
	v_min_date	timestamp with time zone;
	v_count		integer;
	v_hours		numeric;
BEGIN
    -- Determine starting date for deleting entries
    SELECT min(day) INTO v_min_date FROM timesheet_import;

    -------------------------------------------------------------------
    -- Deal with removed timesheet entries.
    -- Search for those TS entries that are present in im_hours, but
    -- not present (anymore...) in timesheet_import.
    RAISE NOTICE 'timesheet-import-routine.sql: Starting to delete entries';
    FOR row IN
	select
		h.day, h.project_id, h.user_id, h.cost_id
	from
		im_hours h,
		im_projects p,
		users u
	where
		h.day >= v_min_date and
		h.project_id = p.project_id and
		h.user_id = u.user_id and
		not exists (
			select	*
			from	timesheet_import t
			where	p.nav_project_nr = t.navision_project_nr and
				lower(u.username) = lower(t.username) and
				h.day = t.day
		)
    LOOP
	RAISE NOTICE 'timesheet-import-routine.sql: Deleting im_hours entry: user_id=%, project_id=%, day=%', row.user_id, row.project_id, row.day;

	-- Deal with im_costs representing the timesheet cost.
	-- Needs to be deleted before we can remove the im_hours entry.
        UPDATE im_hours SET cost_id = NULL
        WHERE cost_id = row.cost_id;

        PERFORM im_cost__delete(row.cost_id);

	DELETE	from im_hours
	WHERE	day = row.day and
		project_id = row.project_id and
		user_id = row.user_id;
    END LOOP;

	
    -------------------------------------------------------------------
    -- Import actual timesheet records    
    RAISE NOTICE 'timesheet-import-routine.sql: Starting to insert entries';
    FOR row IN
        SELECT * FROM timesheet_import
    LOOP
	v_user_id = timesheet_user__get_id(row.username, row.email);
	IF v_user_id IS NULL THEN 
		RAISE WARNING 'timesheet-import-routine.sql: Did not find username % %', row.username, row.email;
		CONTINUE; 
	END IF;

	SELECT project_id INTO v_project_id FROM im_projects
	WHERE nav_project_nr = row.navision_project_nr;
	IF v_project_id IS NULL THEN 
		RAISE WARNING 'timesheet-import-routine.sql: Did not find project %', row.navision_project_nr;
		CONTINUE; 
	END IF;

--	RAISE NOTICE 'timesheet-import-routine.sql: % %', row.username, row.email;

	SELECT count(*) into v_count from im_hours
	WHERE	user_id = v_user_id and
		project_id = v_project_id and
		"day" = row."day";

	IF v_count = 0 THEN
		INSERT INTO im_hours(
			user_id, project_id, "day", hours, note, internal_note, material_id
		) VALUES (
			v_user_id, v_project_id, row."day", row."hours", NULL, NULL, NULL
		);
	ELSE
		-- Try to avoid updates, because updates trigger the
		-- cost cache, which is a quite slow procedure.
		SELECT	hours into v_hours FROM im_hours
		WHERE	user_id = v_user_id and
			project_id = v_project_id and
			"day" = row."day";

		IF v_hours != row.hours THEN
			RAISE NOTICE 'timesheet-import-routine.sql: % %: Updating existing record', row.username, row.email;
			UPDATE im_hours SET
				hours = row.hours
			WHERE	user_id = v_user_id and
				project_id = v_project_id and
				"day" = row."day";
		END IF;
	END IF;

    END LOOP;
    RETURN 0;
END;$BODY$ LANGUAGE 'plpgsql' VOLATILE;
SELECT timesheet_import_routine();




